IF EXISTS (SELECT * FROM dbo.sysObjects WHERE name = 'tbl_app_ServiceRequest' AND type = 'U')
	DROP TABLE dbo.tbl_app_ServiceRequest
	GO

CREATE TABLE dbo.tbl_app_ServiceRequest
(
    Id              int             NOT NULL	IDENTITY(1, 1)	NOT FOR REPLICATION,
    SId		  int		      NOT NULL,
    Description	  varchar(2000)	NOT NULL,
    RequestType	  char(2)		NOT NULL	DEFAULT ('AN'),
    RequestNo	  varchar(9)	NOT NULL	DEFAULT ('0000-0000'),
    RequestDate	  smalldatetime	NOT NULL	DEFAULT (CONVERT(smalldatetime, CONVERT(varchar(8), getdate(), 112))),
    Requestor	  varchar(255)	NULL,
    SendDate	  smalldatetime	NULL,
    SendBy		  varchar(20)	NULL,
    ConfirmedBy	  varchar(255)	NULL,
    ProjectId	  int		      NOT NULL	DEFAULT (0),
    FollowUp	  int		      NOT NULL	DEFAULT (0),
    InsertUser	  sysname		NOT NULL	DEFAULT (CURRENT_USER),
    InsertDate	  smalldatetime	NOT NULL	DEFAULT (getdate()),
    UpdateUser	  sysname		NULL,
    UpdateDate	  smalldatetime	NULL,

    CONSTRAINT pk_tbl_app_ServiceRequest PRIMARY KEY NONCLUSTERED
    (Id),

    CONSTRAINT fk_tbl_app_Stammdaten FOREIGN KEY (SId)
    REFERENCES dbo.tblStammdaten (SId),

    CONSTRAINT idx_tbl_app_ServiceRequest UNIQUE CLUSTERED
    (RequestType, RequestNo),

    CONSTRAINT chk_tbl_app_ServiceRequest_RequestType
    CHECK (RequestType IN ('AN', 'AU', 'RE'))
)
GO

-- Inserttrigger
CREATE TRIGGER dbo.trg_tbl_app_ServiceRequest_Insert
ON dbo.tbl_app_ServiceRequest
FOR INSERT
NOT FOR REPLICATION
AS
    SET NOCOUNT ON

    DECLARE	@RequestCounter	int
    DECLARE	@RequestNumber	varchar(9)

    -- Ermittlung der nchsten Requestnummer (basierend auf RequestType)
    SELECT	@RequestCounter = ISNULL(COUNT(r.Id), 0) + 1
    FROM	inserted i INNER JOIN dbo.tbl_app_ServiceRequest r
            ON (i.RequestType = r.RequestType)
    WHERE	SUBSTRING(r.RequestNo, 6, 4) = YEAR(getdate())

    SET	@RequestNumber = REPLICATE('0', 4 - LEN(@RequestCounter)) + CONVERT(varchar(4), @RequestCounter)
    SET	@RequestNumber = @RequestNumber + '-' + CONVERT(varchar(4), YEAR(getdate()))

    UPDATE	r
    SET	RequestNo = @RequestNumber
    FROM	inserted i INNER JOIN dbo.tbl_app_ServiceRequest r
            ON (i.Id = r.Id)

    SET NOCOUNT OFF
GO

CREATE TRIGGER dbo.trg_tbl_app_ServiceRequest_Update
ON dbo.tbl_app_ServiceRequest
FOR UPDATE
NOT FOR REPLICATION
AS
	SET NOCOUNT ON

	UPDATE	r
	SET	InsertUser = r.InsertUser,
		  InsertDate = r.InsertDate,
		  UpdateUser = CURRENT_USER,
		  UpdateDate = getdate()
	FROM	inserted i INNER JOIN dbo.tbl_app_ServiceRequest r
		    ON (i.Id = r.Id)

	SET NOCOUNT OFF
GO


-- tbl_app_ServiceRequest_Details
IF EXISTS (SELECT * FROM dbo.sysObjects WHERE name = 'tbl_app_ServiceRequest_Details' AND type = 'U')
	DROP TABLE dbo.tbl_app_ServiceRequest_Details
	GO

CREATE TABLE dbo.tbl_app_ServiceRequest_Details
(
	Id		int		NOT NULL	IDENTITY (1, 1) NOT FOR REPLICATION,
	RequestId	int		NOT NULL,
	Position	tinyint		NOT NULL	DEFAULT (0),
	ArticleNo	int		NOT NULL	DEFAULT (0),
	Description	varchar(4000)	NULL,
	Number		smallint	NOT NULL	DEFAULT (1),
	Unit		varchar(10)	NOT NULL	DEFAULT ('Std'),
	Price		numeric(6, 2)	NOT NULL,
	Discount	numeric(4, 2)	NOT NULL	DEFAULT (0),
	VAT		numeric(4, 2)	NOT NULL	DEFAULT (0.19),
	Accepted	bit		NOT NULL	DEFAULT (0),
	InsertUser	sysname		NOT NULL	DEFAULT (CURRENT_USER),
	InsertDate	datetime	NOT NULL	DEFAULT (getdate()),
	UpdateUser	sysname		NULL,
	UpdateDate	datetime	NULL,

	CONSTRAINT pk_tbl_app_ServiceRequest_Details PRIMARY KEY NONCLUSTERED
	(Id),

	CONSTRAINT idx_tbl_app_ServiceRequest_Details UNIQUE CLUSTERED
	(RequestId, Position),

	CONSTRAINT fk_tbl_app_ServiceRequest_Details FOREIGN KEY
	(RequestId) REFERENCES dbo.tbl_app_ServiceRequest (Id)
	ON DELETE CASCADE
)
GO

-- InsertTrigger
CREATE TRIGGER dbo.trg_tbl_app_ServiceRequest_Details_Insert
ON dbo.tbl_app_ServiceRequest_Details
FOR INSERT
NOT FOR REPLICATION
AS
	SET NOCOUNT ON

	UPDATE	d
	SET	d.Discount = CASE WHEN i.Discount >= 1
				  THEN i.Discount / 100
				  ELSE i.Discount
			     END,
		d.VAT      = CASE WHEN i.VAT >= 1
				  THEN i.VAT / 100
				  ELSE i.VAT
			     END
	FROM	inserted i INNER JOIN dbo.tbl_app_ServiceRequest_Details d
		ON (i.Id = d.Id)
	WHERE	i.Discount >= 1 OR
		i.VAT	   >= 1

	SET NOCOUNT OFF
GO

-- UpdateTrigger
CREATE TRIGGER dbo.trg_tbl_app_ServiceRequest_Details_Update
ON dbo.tbl_app_ServiceRequest_Details
FOR UPDATE
NOT FOR REPLICATION
AS
	SET NOCOUNT ON

	UPDATE	s
	SET	s.Discount = CASE WHEN i.Discount >= 1
				  THEN i.Discount / 100
				  ELSE i.Discount
			     END,
		s.VAT      = CASE WHEN i.VAT >= 1
				  THEN i.VAT / 100
				  ELSE i.VAT
			     END,
		s.InsertUser = d.InsertUser,
		s.InsertDate = d.InsertDate,
		s.UpdateUser = CURRENT_USER,
		s.UpdateDate = getdate()
	FROM	dbo.tbl_app_ServiceRequest_Details s INNER JOIN deleted d
		ON (s.Id = d.Id) INNER JOIN inserted i
		ON (s.ID = i.Id)

	SET NOCOUNT OFF
GO